#!/usr/bin/env python
# -*- coding: utf-8 -*-

from config import Config
from display_img import DisplayImg
import numpy as np
import cv2
import os

class ImgProcessor:
    """!
    @brief Performs alpha blending and gamma correction.

    This class provides methods for image processing, including alpha blending 
    of overlapping image regions and gamma correction for brightness adjustment.
    """

    def __init__(self, config):
        """!
        @brief Initializes the ImgProcessor object.

        Sets the configuration object to retrieve parameters like overlap width 
        and gamma values.

        @param config Configuration object containing parameters.
        """
        self.__config = config

    def alphaBlend(self, l_img, r_img):
        """!
        @brief Performs alpha blending on overlapping regions of two images.

        Blends the overlapping regions of the left and right images using 
        a linear gradient alpha mask.

        @param l_img Left image.
        @param r_img Right image.
        @return Tuple containing processed left and right images.
        """
        overlap_width = self.__config.getOverlapWidth()
        
        l_overlap = l_img[:, -overlap_width:, :].copy()
        r_overlap = r_img[:, :overlap_width, :].copy()

        height = l_overlap.shape[0]

        alpha = np.linspace(1, 0, overlap_width).reshape(1, overlap_width, 1)
        alpha = np.tile(alpha, (height, 1, 3))

        l_overlap = (alpha * l_overlap).astype(np.uint8)
        r_overlap = ((1 - alpha) * r_overlap).astype(np.uint8)

        l_common = l_img[:, :-overlap_width, :]
        r_common = r_img[:, overlap_width:, :]
        l_processed = np.concatenate([l_common, l_overlap], axis=1)
        r_processed = np.concatenate([r_overlap, r_common], axis=1)

        return l_processed, r_processed

    def GammaCorrection(self, img):
        """!
        @brief Applies gamma correction to an image.

        Adjusts the brightness of the image using gamma correction based 
        on the gamma value in the configuration.

        @param img Input image.
        @return Gamma-corrected image.
        """
        gamma = self.__config.getGamma()
        inv_gamma = 1.0 / gamma
        lookup_table = np.array([((i / 255.0) ** inv_gamma) * 255 for i in range(256)]).astype("uint8")
        corrected_image = cv2.LUT(img, lookup_table)
        return corrected_image

    def cropImage(self):
        """!
        @brief Crops the left and right halves of an image with overlap.

        Reads the image from the specified path, splits it into left and 
        right halves with the overlap region, and returns the cropped images.

        @return Tuple containing cropped left and right images.
        """
        path = str(os.getcwd() + self.__config.getImgPath())
        overlap_width = self.__config.getOverlapWidth()
        image = cv2.imread(path)
        width = image.shape[1]
        l_img = image[:, :width // 2 + overlap_width, :]
        r_img = image[:, width // 2 - overlap_width:, :]
        return l_img, r_img
