#!/usr/bin/env python
# -*- coding: utf-8 -*-
from config import Config
from img_processor import ImgProcessor
from display_img import DisplayImg
import cv2

class MainStitcher:
    """!
    @brief A class responsible for stitching and displaying images in a projection system.

    The class integrates configuration settings from the `Config` class and display properties
    from the `DisplayImg` class to handle the main stitching and display logic.
    """

    def __init__(self, dImg):
        """!
        @brief Initializes the MainStitcher with configuration and display image settings.
        @param config An instance of the Config class containing projection configuration.
        @param dImg An instance of the DisplayImg class containing display settings.
        """
        self.__displayImg = dImg


    #for single monitor/mirroring
    def singleDisplay(self):
        """!
        @brief use two laptops to project the final processed images.
        """
        img = self.__displayImg.getImg()
        side = self.__displayImg.getSide()

        # cv2.namedWindow(side, cv2.WINDOW_NORMAL)
        cv2.imshow(side, img)
        # cv2.setWindowProperty(side, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)


    #for double extended monitor
    def doubleDisplay(self, monitorWidth):
        """!
        @brief use two laptops to project the final processed images.
        """
        img = self.__displayImg.getImg()
        side = self.__displayImg.getSide()

        """
        In moveWindow(name, x, y), we can replace x with the window size or something
        """
        if(side == "Left"):
            cv2.namedWindow(side, cv2.WINDOW_NORMAL)
            cv2.moveWindow(side, 0, 0)
            cv2.imshow(side, img)
            cv2.resizeWindow(side, monitorWidth, 1080)
            cv2.setWindowProperty(side, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)

        elif(side == "Right"):
            cv2.namedWindow(side, cv2.WINDOW_NORMAL)
            cv2.moveWindow(side, monitorWidth, 0)
            cv2.imshow(side, img)
            cv2.resizeWindow(side, monitorWidth, 1080)
            cv2.setWindowProperty(side, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)

    def save(self):
        cv2.imwrite("./img/processed/" + self.__displayImg.getSide() + "_img.png", self.__displayImg.getImg())


def main():
    config = Config.readConfigFile()

    imgProcessor = ImgProcessor(config)

    l_img, r_img = imgProcessor.cropImage()
    l_alpha_processed, r_alpha_processed = imgProcessor.alphaBlend(l_img, r_img)


    l_alpha_gamma_processed = imgProcessor.GammaCorrection(l_alpha_processed)
    r_alpha_gamma_processed = imgProcessor.GammaCorrection(r_alpha_processed)

    l_displayImg = DisplayImg(l_alpha_gamma_processed.shape[1] ,l_alpha_gamma_processed.shape[0], config.getOverlapWidth(), l_alpha_gamma_processed, "Left")
    r_displayImg = DisplayImg(r_alpha_gamma_processed.shape[1] ,r_alpha_gamma_processed.shape[0], config.getOverlapWidth(), r_alpha_gamma_processed, "Right")

    l_stitcher = MainStitcher(l_displayImg)
    r_stitcher = MainStitcher(r_displayImg)


    print(config.getIsDualMonitor())

    if config.getIsDualMonitor():
        l_stitcher.doubleDisplay(config.getMonitorWidth())
        r_stitcher.doubleDisplay(config.getMonitorWidth())
    else:
        if config.getSide().lower() == "right":
            r_stitcher.singleDisplay()
        else:
            l_stitcher.singleDisplay()

    cv2.waitKey(0)
    cv2.destroyAllWindows()


    l_stitcher.save()
    r_stitcher.save()

if __name__ == "__main__":
    main()